/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Istex
*  fichier  : IstexAffiliationSetPays.c 
*  Auteur   : Jacques DUCLOY
*  Date     : fin 2013
*
***********************************************************************/

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */

#include <getopt.h>
#include "SxPath.h"
#include "Buffer.h"
#include "WicriPaysFromEn.h"
#include <ctype.h>

SxPathResult *teiPathAffiliation;
SxPathResult *teiPathAuthor;
SxPathResult *teiPathAffiliationOld;

SxmlNode *docInput;
SxmlNode * affiliation;
SxmlNode *docInput;
SxmlNode *listAffiliations;
SxmlNode *listAuthors;

char *lastCountryByAuthor;
SxmlNode *nextAffiliation;

Buffer* bufPays;
char *inputAffiliationStr;
SxmlNode *countryNode;

char *proceedAffiliation()
{
  SxmlNode *inputAffiliation;
  char *beginAff;
  char *posComma;
  static Buffer *bufSubField=NULL;
  static Buffer *bufField=NULL;
  static Buffer *bufArea=NULL;
  char *beginSubField;

  if (!bufSubField) bufSubField=NewBuffer();
  inputAffiliation=SxmlGetFirstChildByTagName(affiliation,"mods:affiliation");
  if(!inputAffiliation)return NULL;
  inputAffiliationStr=SxmlLeafText(inputAffiliation);
  if (!(beginAff=inputAffiliationStr))
    { 
      SxmlAppendChild(affiliation, WicriGenerNoCountry("???","syntax"));
      return NULL;
    }
  posComma=strrchr(beginAff, ',');
  if (!posComma)
    {
      if ((countryNode=WicriAffiliationSetPaysFromSubFieldEn(affiliation,  beginAff))) return SxmlLeafText(countryNode);
      if ((!lastCountryByAuthor)&&(!nextAffiliation)) SxmlAppendChild(affiliation, WicriGenerNoCountry(beginAff,"no comma"));
      return NULL;
    }
  beginSubField=posComma+1;
  while (beginSubField[0]==' ') beginSubField++;
  if ( (beginSubField[0]=='(')&& (beginSubField[strlen(beginSubField)-1]==')'))
    {
      BufferStrcpy(bufSubField, beginSubField+1);
      BufferTailCut(bufSubField,1);
    }
  else if ( (beginSubField[strlen(beginSubField)-1]=='.') && (islower(beginSubField[strlen(beginSubField)-2])))
    {
      BufferStrcpy(bufSubField, beginSubField);
      BufferTailCut(bufSubField,1);
    }
  else  BufferStrcpy(bufSubField, beginSubField);
  countryNode=WicriAffiliationSetPaysFromSubFieldEn(affiliation,  BufferString(bufSubField));
  if (!countryNode)    /* perhaps sth like , Nancy - France */
    {
      char *posHyphen;
      posHyphen=strrchr(beginAff, '-');
      if ((posHyphen > posComma))
	{
	  BufferStrcpy(bufSubField, posHyphen+2);
	  countryNode=WicriAffiliationSetPaysFromSubFieldEn(affiliation,  BufferString(bufSubField));
	}
    }
  if (!countryNode)    /* perhaps sth like , Nancy  (France) */
    {
      char *posPar;
      posPar=strrchr(beginAff, '(');
     if ((posPar > posComma))
	{
	  char *posPar2;
	  posPar2=strchr(posPar,')');
	  if (posPar2)
	    {
	      BufferStrncpy(bufSubField, posPar+1, posPar2-posPar-1);
	      countryNode=WicriAffiliationSetPaysFromSubFieldEn(affiliation,  BufferString(bufSubField));
	    }
	}
    }
  if (!countryNode)    /* perhaps sth like , Nancy France */
    {
      char *posSpace;
      posSpace=strrchr(beginAff, ' ');
      if ((posSpace > posComma))
	{
	  BufferStrcpy(bufSubField, posSpace+1);
	  countryNode=WicriAffiliationSetPaysFromSubFieldEn(affiliation,  BufferString(bufSubField));
	}
    }
  if (!countryNode)
    {
      char *univ;
      univ=strstr(inputAffiliationStr,"Univ");
      /*      countryNode=WicriSetPaysOrgFromUniv(affiliation, univ); */
      countryNode=WicriAffiliationSetPaysFromFieldEn(affiliation, inputAffiliationStr);
      if (countryNode) SxmlSetAttribute(affiliation, "wicri:level", "4");
    }
  if ((countryNode))
    {
      char *level;
      level=SxmlGetAttribute(affiliation, "wicri:level");
      if(!bufArea)bufArea=NewBuffer();
      BufferStrncpy(bufArea, beginAff, posComma-beginAff);
      switch(level[0])
	{
	case '1':
	  SxmlAppendChild(affiliation, SxmlLeafCreate("wicri:regionArea", BufferString(bufArea)));
	  break;
	case '2':
	  SxmlAppendChild(affiliation, SxmlLeafCreate("wicri:cityArea", BufferString(bufArea)));
	  break;
	case '3':
	  SxmlAppendChild(affiliation, SxmlLeafCreate("wicri:orgArea", BufferString(bufArea)));
	  break;
	default:
	  break;
	}
      return SxmlLeafText(countryNode);
    }

  if (!bufField) bufField=NewBuffer();
  BufferStrcpy(bufField,beginAff);
  if (BufferTailCmp(bufField, "and")==0)
    {
      BufferTailCut(bufField,4);
      countryNode=WicriAffiliationSetPaysFromFieldEn(affiliation, BufferString(bufField));
      if (countryNode)return SxmlLeafText(countryNode);
    }
  
  
  if ((!lastCountryByAuthor)&&(!nextAffiliation)) SxmlAppendChild(affiliation, WicriGenerNoCountry(BufferString(bufSubField),"subField"));
  return NULL;
}

void proceedDoc()
{
  if (!SxmlNodeHasName(docInput,"record")) return;
  lastCountryByAuthor=NULL;
  if ((listAffiliations=SxPathSetResultListCreate(teiPathAffiliationOld, docInput))) /* to deprecate */
    {
      while((affiliation=SxmlNodeListNextNode(listAffiliations)))
	{
	  char *country;
	  country=proceedAffiliation();
	  if (country)
	    {
	      if (strcmp(country, "Yougoslavie")==0)
		{
		  if (strstr(inputAffiliationStr, "Beograd"))  SxmlLeafSetText(countryNode, "Serbie");
		  else if (strstr(inputAffiliationStr, "Zagreb"))  SxmlLeafSetText(countryNode, "Croatie");
		  else if (strstr(inputAffiliationStr, "Ljubljana"))  SxmlLeafSetText(countryNode, "Slovénie");
		}
	    }
	}
    }
  /*  if ((listAffiliations=SxPathSetResultListCreate(teiPathAffiliation, docInput))) */
  if ((listAuthors=SxPathSetResultListCreate(teiPathAuthor, docInput))) 
    {
      SxmlNode *author;
      /*      while((affiliation=SxmlNodeListNextNode(listAffiliations))) */
      while((author=SxmlNodeListNextNode(listAuthors))) 
	{
	  lastCountryByAuthor=NULL;
	  affiliation=SxmlGetFirstChildByTagName(author, "affiliation");
	  nextAffiliation=NULL;
	  while (affiliation)
	    {
	      char *country;
	      nextAffiliation=SxmlGetNextSiblingByTagName(affiliation, "affiliation");
	      country=proceedAffiliation();
	      if (country)
		{
		  lastCountryByAuthor=country;
		  if (strcmp(country, "Yougoslavie")==0)
		    {
		      if (strstr(inputAffiliationStr, "Beograd"))  SxmlLeafSetText(countryNode, "Serbie");
		      else if (strstr(inputAffiliationStr, "Zagreb"))  SxmlLeafSetText(countryNode, "Croatie");
		      else if (strstr(inputAffiliationStr, "Ljubljana"))  SxmlLeafSetText(countryNode, "Slovénie");
		    }
		}
	      affiliation=nextAffiliation;
	    }
	}
    }
}

int main () {
  bufPays=BufferCreate(10,10);
  teiPathAffiliationOld=SxPathSetCompile("TEI/teiHeader/fileDesc/titleStmt/author/affiliation");
  teiPathAffiliation=SxPathSetCompile("TEI/teiHeader/fileDesc/sourceDesc/biblStruct/analytic/author/affiliation");
  teiPathAuthor=SxPathSetCompile("TEI/teiHeader/fileDesc/sourceDesc/biblStruct/analytic/author");

  while ((docInput=SxmlInputNextDocumentElement()))
    {      
      proceedDoc();
      SxmlPrint (SxmlInputRecord);
    }
   exit(EXIT_SUCCESS);
}
