/*   -*- coding: utf-8 -*-  */

/*
  Set of tools for cleaning restricted access data from Istex elements 
 */

#include "Istex.h"
#include "StrDict.h"

StrDict *IstexMetadataXmlCleanRules=NULL;
Buffer  *IstexBufMessClean=NULL;
Buffer  *IstexCleanBufKey=NULL;
char *IstexCleanHfdKey;
int   IstexCleanTrace;

void IstexCleanMetadataXmlInit()
{
  Buffer *bufPath;
  bufPath=NewBuffer();
  BufferStrcpy(bufPath, getenv("DILIB"));
  BufferStrcat(bufPath, "/data/Istex/IstexMetadataXmlCleanRules.tab");
  IstexMetadataXmlCleanRules=StrDictFromFile(BufferString(bufPath));
  IstexBufMessClean=NewBuffer();
  IstexCleanBufKey=NewBuffer();
  IstexCleanHfdKey=NULL;
  IstexCleanTrace=0;
}

void IstexCleanMetadataSetAbn(SxmlNode *metaXml)
{
  SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
  if(IstexCleanHfdKey) SxmlSetAttribute (metaXml, "wicri:rec", IstexCleanHfdKey);
}

SxmlNode *IstexCleanMetadataXmlIterNode(SxmlNode *metaXml, SxmlNode *documentNode)
{
  SxmlNode *item;
  SxmlNode *retNode;
  BufferStrcat(IstexCleanBufKey,"/");
  item=SxmlFirstChild(documentNode);
  while(item)
    {
      SxmlNode *nextItem;
      char *itemRule;
      int lenName;
      char *itemName;
      nextItem=SxmlNextSibling(item);   /*   for next if deleted */
      itemName=SxmlNodeName(item);
      lenName=strlen(itemName);
      BufferStrcat(IstexCleanBufKey, itemName);
      itemRule=StrDictSearch(IstexMetadataXmlCleanRules, BufferString(IstexCleanBufKey));
      if (!itemRule)
	{
	  BufferStrcat(IstexBufMessClean, ", element ");
	  BufferStrcat(IstexBufMessClean, itemName);
	  BufferStrcat(IstexBufMessClean, " not found");
	  SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
	  return NULL;
	}
      switch(itemRule[0])
	{
	case 'r':
	  BufferStrcat(IstexBufMessClean, " done");
	  SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
	  return metaXml;
	case 'i':
	  if (IstexCleanTrace)
	    {
	      BufferStrcat(IstexBufMessClean, ", ");
	      BufferStrcat(IstexBufMessClean, itemName);
	    }
	  retNode=IstexCleanMetadataXmlIterNode(metaXml, item);
	  if (!retNode)return NULL;
	  break;
	case 's':
	  if (IstexCleanTrace)
	    {
	      BufferStrcat(IstexBufMessClean, ", ");
	      BufferStrcat(IstexBufMessClean, itemName);
	      BufferStrcat(IstexBufMessClean, "  saved");
	    }
	  break;
	case 'k':
	  if (IstexCleanTrace)
	    {
	      BufferStrcat(IstexBufMessClean, ", ");
	      BufferStrcat(IstexBufMessClean, itemName);
	      BufferStrcat(IstexBufMessClean, "  killed");
	    }
	  SxmlFree(item);
	  break;
	}
      BufferTailCut(IstexCleanBufKey,lenName);
      item=nextItem;
    }
  BufferTailCut(IstexCleanBufKey,1);
  return documentNode;
}

SxmlNode *IstexCleanMetadataXml(SxmlNode *metaXml, char *corpusName)
{
  SxmlNode *document;
  char *corpusRule;
  SxmlNode *retNode;

  document=SxmlGetFirstChildByTagName(metaXml,"istex:document");
  BufferStrcpy(IstexBufMessClean, "corpus ");
  BufferStrcat(IstexBufMessClean, corpusName);
  corpusRule=StrDictSearch(IstexMetadataXmlCleanRules, corpusName);
  if (!corpusRule) 
    {
       BufferStrcat(IstexBufMessClean, " not found");
       IstexCleanMetadataSetAbn(metaXml);
       return NULL;
    }
  if (corpusRule[0]=='r')
    {
      BufferStrcat(IstexBufMessClean, " done");
      SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
      return metaXml;
    }
  if (corpusRule[0]=='i')
    {
      BufferStrcpy(IstexCleanBufKey,corpusName);
      retNode=IstexCleanMetadataXmlIterNode(metaXml, document);
      SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
      return retNode;
    }
  else SxmlSetAttribute (metaXml, "rule", corpusRule);
  SxmlSetAttribute (metaXml,"wicri:clean", BufferString(IstexBufMessClean));
  return metaXml;
}


SxmlNode *IstexCleanMetadataMods(SxmlNode *metaMods, char* corpusName)
{
  SxmlNode *relatedItem;
  relatedItem=SxmlGetFirstChildByTagName(metaMods, "relatedItem");
  while (relatedItem)
    {
      SxmlNode *nextItem;
      nextItem=SxmlGetNextSiblingByTagName(relatedItem, "relatedItem");
      if (SxmlHasAttribute(relatedItem, "type", "isReferencedBy")) SxmlFree(relatedItem);
      relatedItem=nextItem;
    }
  return metaMods;
}
