/*   -*- coding: utf-8 -*-  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Istex.h"

extern char *optarg;
extern int   optind;
int getopt();

int   outputType;
char *outputTypeParameter;
char *jsonOutput;

char idType; /*   a: ark  
                  i: istex id
                  d: doi */

void printDocument(char *id)
{
  char *url;
  char *url2;
  char *resultStr;
  static SxmlNode *resultNode=NULL;
  static SxmlNode *jsonNode=NULL;

  if (IstexTraceLevel >0) printf("======= %s ======\n", id);
  switch (outputType)
    {
    case 'j':
      switch (idType)
	{
	case 'i':
	  url=IstexGetIdQueryUrl(id, jsonOutput);
	  break;
	case 'd':
	  url=IstexGetDoiQueryUrl(id, jsonOutput);
	  break;
	}
      if (url) printf("%s\n", IstexGetDocumentByUri(url));
      return;
    case 'J':
      url=IstexGetIdQueryUrl(id, jsonOutput);
      resultStr=IstexGetDocumentByUri(url);
      SxmlPrint(JsonToSxml(resultStr));
      putchar('\n');
      return;
    case 'a':
    case 'A':
    case 'M':
    case 'O':
    case 'o':
    case 'u':
    case 'U':
      switch (idType)
	{
	case 'i':
	  url=IstexGetIdQueryUrl(id, "*");
	  break;
	case 'd':
	  url=IstexGetDoiQueryUrl(id, "*");
	  break;
	}
     
      if (!url) return;
      resultStr=IstexGetDocumentByUri(url);
      if (jsonNode)SxmlFree(jsonNode);
      jsonNode=JsonToSxml(resultStr);
      break;
    }

  if (resultNode) SxmlFree(resultNode);
  switch (outputType)
    {
    case 'M':
      resultNode=IstexGetFullDocFromJson (jsonNode, 0);
      SxmlPrint(resultNode);
      putchar ('\n');
      return;
    case 'a':
      resultNode=IstexGetFullDocFromJson (jsonNode, 1);
      SxmlPrint(resultNode);
      putchar ('\n');
      return;
    case 'A':
      resultNode=IstexGetFullDocFromJson (jsonNode, 2);
      SxmlPrint(resultNode);
      putchar ('\n');
      return;
    case 'u':
      url2=IstexJsonFindUrl(jsonNode, "metadata", outputTypeParameter);
      if (url2) printf ("%s\n", url2);
      return;
    case 'U':
      url2=IstexJsonFindUrl(jsonNode, "fulltext", outputTypeParameter);
      if (url2) printf ("%s\n", url2);
      return;
    case 'o':
      url2=IstexJsonFindUrl(jsonNode, "metadata", outputTypeParameter);
      break;
    case 'O':
      url2=IstexJsonFindUrl(jsonNode, "fulltext", outputTypeParameter);
      break;
    }

  switch (outputType)
    {
    case 'o':
      switch(*outputTypeParameter)
	{
	case 'm':            /*   MODS */
	  resultNode=IstexGetDocumentModsByUri(url2, id);
	  if (resultNode)
	    {
	      SxmlPrint(resultNode);
	      putchar('\n');
	    }
	  return;
	case 'x':         /*  XML */
	case 'X':         /*  xml */
	  resultNode=IstexGetDocumentXmlByUri(url2);
	  break;
	}
    case 'O':
      switch(*outputTypeParameter)
	{
	case 't':         /*  TEI */
	case 'T':         /*  TEI */
	  resultNode=IstexGetDocumentTeiByUri(url2, "TEI");
	  break;
	}
      SxmlPrint(resultNode);
      putchar ('\n');
      break;
    }
}


int main (int argc, char **argv)
{
  int cOption;
  int stdinFlag;
  int option_I;
  char *tokenParam;
  Buffer *inputId;

  IstexInit();
  stdinFlag=1;
  option_I=0;
  inputId=NewBuffer();
  outputType='j';
  tokenParam=NULL;
  jsonOutput=NULL;
  idType='i';

  while((cOption=getopt(argc,argv,"aAci:j:k:J:IMo:O:t:T:u:U:v:"))!=EOF)
    {switch (cOption)
	{
	case 'a':
	  outputType='a';
	  break;
	case 'A':
	  outputType='A';
	  break;
	case 'c':
	  IstexCleanFlag=1; 
	  break;
	case 'M':
	  outputType='M';
	  break;
	case 'i':
	  idType=optarg[0];
	  break;
	case 'I':
	  option_I=1;
	case 'k':
	  printDocument (optarg);
	  stdinFlag=0;
	  break;
	case 'j':
	  outputType='j';
	  jsonOutput=optarg;
	  break;
	case 'J':
	  outputType='J';
	  jsonOutput=optarg;
	  break;
	case 'o':
	  outputType='o';
	  outputTypeParameter=optarg;
	  break;
	case 'O':
	  outputType='O';
	  outputTypeParameter=optarg;
	  break;
	case 't':
	  tokenParam=optarg;
	  IstexSetAuthToken(tokenParam);
	  break;
	case 'u':
	  outputType='u';
	  outputTypeParameter=optarg;
	  break;
	case 'U':
	  outputType='U';
	  outputTypeParameter=optarg;
	  break;
	case 'T':
	  IstexTraceLevel=atoi(optarg);
	  break;
	case 'v':
	  IstexRootUrl=optarg;
	  break;
	}
    }

  if (option_I==1)stdinFlag=1;

  if (stdinFlag==0)
    {
      IstexEnd();
      exit(EXIT_SUCCESS);
    }

  while (BufferGets(inputId))
    {
      printDocument (BufferString(inputId));
    }
  IstexEnd();
  exit(EXIT_SUCCESS);
}

