/*   -*- coding: utf-8 -*-  */

/**********************************************************************
*
*  module   : Istex
*  fichier  : IstexRecordTools.c
*  Auteur   : Jacques DUCLOY
*  Date     : 2013
*
***********************************************************************/

#include "IstexRecord.h"
#include "TeiHandler.h"
#include "SxPath.h"
#include "SxmlNode.h"
#include "Buffer.h"
#include <ctype.h>

 SxmlNode *IstexRecordTeiRoot;
 SxmlNode *IstexRecordTeiHeader;
 SxmlNode *IstexRecordTeiFileDesc;
 SxmlNode *IstexRecordTeiSourceDesc;
 SxmlNode *IstexRecordTeiProfileDesc;

 SxmlNode *IstexRecordEnrichments;

 SxmlNode *IstexRecordModsRoot;

 char     *IstexRecordLanguage;

SxPathResult *IstexPathTeeft=NULL;

SxmlNode *IstexRecordGetModsElement(SxmlNode *inputDoc)
{
  SxmlNode *metaNode;
  SxmlNode *modsRoot; 

  metaNode=SxmlGetFirstChildByTagName(inputDoc,"metadata");
  while (metaNode)
    {
      if ((modsRoot=SxmlGetFirstChildByTagName(metaNode,"mods"))) 
	{
	  IstexRecordModsRoot=modsRoot;
	  return modsRoot;
	}
      metaNode=SxmlGetNextSiblingByTagName(metaNode,"metadata");
    }
  return NULL;
}

SxmlNode *IstexRecordGetTeiElement(SxmlNode *inputDoc)
{
  SxmlNode *fullTextNode;
  fullTextNode=SxmlGetFirstChildByTagName(inputDoc,"fulltext");
  while (fullTextNode)
    {
      if ((IstexRecordTeiRoot=SxmlGetFirstChildByTagName(fullTextNode,"istex:fulltextTEI")))
	{
	  IstexRecordTeiHeader=SxmlGetFirstChildByTagName(IstexRecordTeiRoot,"teiHeader");
	  if (IstexRecordTeiHeader)
	    {
	      IstexRecordTeiFileDesc=SxmlGetFirstChildByTagName(IstexRecordTeiHeader,"fileDesc");	     
	      if(IstexRecordTeiFileDesc)IstexRecordTeiSourceDesc=SxmlGetFirstChildByTagName(IstexRecordTeiFileDesc,"sourceDesc");

	      IstexRecordTeiProfileDesc=SxmlGetFirstChildByTagName(IstexRecordTeiHeader,"profileDesc");
	    }
	  return IstexRecordTeiRoot;
	}
      fullTextNode=SxmlGetNextSiblingByTagName(fullTextNode,"fulltext");
    }
  return NULL;
}

char *IstexComputeLangUsageFromTei()
{
  SxmlNode *langUsageIstex;
  SxmlNode *languageIstex;
  if (IstexRecordTeiProfileDesc)
    {
      langUsageIstex=SxmlGetFirstChildByTagName(IstexRecordTeiProfileDesc,"langUsage");
      if (langUsageIstex)
	{
	  languageIstex=SxmlGetFirstChildByTagName(langUsageIstex,"language");
	  if (languageIstex)
	    {
	      char *language;
	      language=SxmlLeafText(languageIstex);
	      if (language && *language)
		{
		  IstexRecordLanguage=language;
		  SxmlAppendChild (Tei_profileDesc, SxmlClone(langUsageIstex));
		}
	    }
	}
    }
  return IstexRecordLanguage;
}

void IstexComputeKeywordsFromTei()
{
  SxmlNode *textClassIstex;
  if (!IstexRecordTeiProfileDesc) return;
  textClassIstex=SxmlGetFirstChildByTagName(IstexRecordTeiProfileDesc,"textClass");
  while (textClassIstex)
    {
      if ((SxmlHasAttribute(textClassIstex, "xml:lang", "en")))
	{
	  SxmlNode *keywordsIstex;
	  if ((keywordsIstex=SxmlGetFirstChildByTagName(textClassIstex, "keywords")))
	    {
	      if (SxmlHasAttribute(keywordsIstex, "scheme", "keyword"))
		{
		  SxmlNode *listIstex;
		  if ((listIstex=SxmlGetFirstChildByTagName(keywordsIstex, "list")))
		    {
		      SxmlNode *itemIstex;
		      SxmlReset(listIstex);
		      while ((itemIstex=SxmlNextNode(listIstex)))
			{
			  if (SxmlNodeHasName(itemIstex, "item"))
			    {
			      char *newKw;
			      newKw=SxmlLeafText(SxmlFirstChild(itemIstex));
			      TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", newKw);
			    }
			}
		    }
		}
	    }
	}
      
      textClassIstex=SxmlGetNextSiblingByTagName(textClassIstex, "textClass");      
    }
}

void IstexComputeKeywordsFromEnrichments()
{
  SxmlNode *teeftRoot;
  SxmlNode *kwSet;
  SxmlNode *kw;

  if (!IstexRecordEnrichments)return;
  teeftRoot=SxmlGetFirstChildByTagName(IstexRecordEnrichments, "teeft");
  if (teeftRoot)
    {
      if (!IstexPathTeeft) IstexPathTeeft=SxPathSetCompile("TEI/standOff/listAnnotation/annotationBlock/keywords/term");
      kwSet=SxPathSetResultListCreate(IstexPathTeeft, teeftRoot);
      if (kwSet)
	{
	  static Buffer* bufTerm=NULL;
	  if (!bufTerm) bufTerm=NewBuffer();
	  SxmlReset(kwSet);
	  while ((kw=SxmlNodeListNextNode(kwSet)))
	    {
	      SxmlNode *termNode;
	      SxmlNode *fsNode;
	      SxmlNode *fNode;
	      SxmlNode *numericNode;
	      char *termValue;
	      char *valueNumStr;
	      double valueNum;
	      if ((termNode=SxmlGetFirstChildByTagName(kw, "term"))
		  && (fsNode=SxmlGetFirstChildByTagName(kw, "fs")) 
		  && (fNode=SxmlGetFirstChildTagAtt(fsNode, "f", "name", "specificity"))
		  && (numericNode=SxmlGetFirstChildByTagName(fNode, "numeric")) 
		  && (valueNumStr=SxmlGetAttribute(numericNode,"value"))
		  )
		{
		  termValue=SxmlLeafText(termNode);
		  valueNum=atof(valueNumStr);
		  if ((valueNum>0.02)
		      || (strchr( BufferString(bufTerm), ' ') && (valueNum>0.01)))
		    {
		      BufferStrcpy(bufTerm,termValue);
		      BufferString(bufTerm)[0]=toupper( BufferString(bufTerm)[0]);
		      /* TeiKeywordsPutTerm("KwdEn", NULL, NULL, "en", BufferString(bufTerm)); */
		      TeiKeywordsPutTerm("Teeft", NULL, NULL, "en", BufferString(bufTerm));
		    }
		}
	    }
	  SxmlFree(kwSet);
	}
    }
}


void IstexRecordInitFromInput(SxmlNode *inputDoc)
{
  IstexRecordLanguage=NULL;
  IstexRecordModsRoot=NULL;
  IstexRecordTeiRoot=NULL;
  IstexRecordTeiHeader=NULL;
  IstexRecordTeiFileDesc=NULL;
  IstexRecordTeiSourceDesc=NULL;
  IstexRecordTeiProfileDesc=NULL;

  IstexRecordEnrichments=SxmlGetFirstChildByTagName (inputDoc, "enrichments");
}
