/*   -*- coding: utf-8 -*-  */

#include "Istex.h"
#include "CharSetBuffer.h"
#include "Except.h"

SxmlNode *IstexGetDocumentModsByUri(char *uri, char*id)
{
  char *istexResult;
  char *posRoot;
  SxmlNode *modsDocument;

  istexResult=IstexGetDocumentByUri(uri);
  posRoot=strstr(istexResult, "<mods");
  modsDocument=SxmlFromString(posRoot);
  if (id) SxmlSetAttribute(modsDocument, "istexId", id);
  if (IstexCleanFlag)IstexCleanMetadataMods (modsDocument, NULL);
  
  return modsDocument;
}

char *IstexFindEndTeiContent(char *beginContent, char *endContent)
{
  while (endContent> beginContent)
    {
      if (*endContent=='<')
	{
	  if(strncmp(endContent, "</TEI",5)==0)return endContent;
	}
      endContent--;
    }
  return NULL;
}

SxmlNode *IstexGetDocumentTeiByUri(char *uri, char *tag)
{
  SxmlNode *teiItem;
  char *beginTEI;
  char *beginTeiContent;
  char *endTeiContent;
  char *istexResult;

  istexResult=IstexGetDocumentByUri(uri);
  /*  teiItem=SxmlElementCreate("istex:fulltextTEI"); */
  teiItem=SxmlElementCreate(tag);
  SxmlSetAttribute(teiItem, "uri", uri);
  SxmlSetAttribute(teiItem, "xmlns:tei", "http://www.tei-c.org/ns/1.0");
  beginTEI=strstr(istexResult, "<TEI");
  if (!beginTEI)
    {
      /*
      fprintf(stderr, "curl failure : %s\n",istexResult);
      exit (EXIT_FAILURE);
      */
      SxmlFree(teiItem);
      return NULL;
    } 
  beginTeiContent=strchr(beginTEI, '>')+1;
  if (*beginTeiContent)
    {
      endTeiContent=strrchr(istexResult, '<');
      if (strncmp(endTeiContent, "</TEI",5)!=0)
	{
	  endTeiContent=IstexFindEndTeiContent(beginTeiContent,endTeiContent-1);
	  if (!endTeiContent)
	    {
	      fprintf(stderr, "**** IstexGetCorpus  : TEI markup problem in %s\n", uri);
	      exit (EXIT_FAILURE);
	    }
	}
      SxmlAppendChild(teiItem, SxmlTextCreateFromSubstr(beginTeiContent, endTeiContent-beginTeiContent));
    }
  return teiItem;
}

SxmlParser *IstexXmlParser=NULL;
SxmlNode *IstexDocXmlWithDtd(char *str)
{
  SxmlNode *istexDoc;
  int codeDOM;

  if(!IstexXmlParser)IstexXmlParser=SxmlParserCreate();
  if(str[0]!='<')
    {
      char *str1;    /* principe de precaution */
      str1=strchr(str, '<'); 
      if (!str1)return NULL;
      str=str1;
    }
  SxmlParserInit(IstexXmlParser, str);
  istexDoc=SxmlElementCreate("istex:metadataXml");
  while ((codeDOM=SxmlParserNextDomType(IstexXmlParser)))
    {
      if (codeDOM==SXML_NODE_PROCESSING_INSTRUCTION)
	{
	  SxmlNode *nodePI;
	  SxmlNode *elemPI;
	  
	  nodePI=SxmlParserGetProcessingInstruction(IstexXmlParser);
	  elemPI=SxmlLeafCreate("istex:xmlDeclaration", SxmlNodeValue(nodePI));
	  SxmlAppendChild(istexDoc, elemPI);
	  SxmlFree(nodePI);
	  SxmlParserSkipSeparator(IstexXmlParser);
	  continue;
	}
      else if (codeDOM==SXML_NODE_DOCUMENT_TYPE)
	{
	  SxmlNode *nodeDtd;
	  SxmlNode *outputDocType;
	  SxmlNode *dtdItem;
	  nodeDtd=SxmlParserGetDocType(IstexXmlParser);
	  SxmlSetNodeType(nodeDtd, SXML_NODE_ELEMENT);
	  SxmlSetNodeName(nodeDtd, strdup ("istex:docType"));
	  SxmlSetAttribute(nodeDtd, "name", SxmlNodeName(nodeDtd));
	  SxmlReset(nodeDtd);
	  while ((dtdItem=SxmlNextNode(nodeDtd)))
	    {
	      char *n1;
	      n1=SxmlNodeName(dtdItem);
	      SxmlSetAttribute(dtdItem, "name", n1);
	      SxmlSetNodeType(dtdItem, SXML_NODE_ELEMENT);
	      SxmlSetNodeName(dtdItem, strdup ("istex:entity"));
	    }
	  SxmlAppendChild(istexDoc, nodeDtd);
	}
      break;
    }
  SxmlParserSkipSeparator(IstexXmlParser);
  SxmlAppendChild(istexDoc, SxmlLeafCreate("istex:document",  SxmlParserContextStr(IstexXmlParser)));
  return istexDoc;
}

SxmlNode *IstexGetDocumentXmlByUri(char *uri)
{
  /* char *posRoot; */
  SxmlNode *metaItem;
  char *istexResult;

  istexResult=IstexGetDocumentByUri(uri);
  metaItem=IstexDocXmlWithDtd(istexResult);
  return metaItem;
}

SxmlNode *IstexGetFullDocFromJson (SxmlNode *jsonDoc, int fulltextTag)
{
  SxmlNode *outputDoc;
  SxmlNode *hit1;
  SxmlNode *istexElem;
  char *xmlnsJson;

  hit1=SxmlFirstChild(SxmlGetFirstChildByTagName(jsonDoc,"hits"));
  if (!hit1)return NULL;

  outputDoc=SxmlElementCreate ("istex");
  xmlnsJson=SxmlGetAttribute(jsonDoc, "xmlns:json");
  if (xmlnsJson) SxmlSetAttribute(outputDoc, "xmlns:json", xmlnsJson);
  SxmlSetAttribute(outputDoc, "xmlns:istex", "http://ticri.univ-lorraine.fr/wicri-meta.fr/index.php/IstexWicri");

  SxmlReset (hit1);

  while ((istexElem=SxmlNextNode(hit1)))
    {
      if (SxmlNodeHasName(istexElem, "metadata"))
	{
	  SxmlNode *jsItem;
	  SxmlNode *metadata;
	  SxmlAppendChild (outputDoc, metadata=SxmlElementCreate("metadata"));
	  SxmlReset(istexElem);
	  while ((jsItem=SxmlNextNode(istexElem)))
	    {
	      SxmlNode *typeNode;
	      SxmlNode *uriNode;
	      SxmlNode *metaItem;
	      char *type;
	      char *uri;

	      typeNode=SxmlGetFirstChildByTagName(jsItem,"extension");
	      if(!typeNode) typeNode=SxmlGetFirstChildByTagName(jsItem,"mimetype");
	      type=SxmlLeafText(typeNode);
	      uriNode=SxmlGetFirstChildByTagName(jsItem, "uri");
	      uri=SxmlLeafText(uriNode);
	      if (strcmp(type, "mods")==0)
		{
		  metaItem=IstexGetDocumentModsByUri(uri, NULL);
		  SxmlAppendChild(metadata, metaItem);
		}
	      else if ((strcmp(type, "xml")==0)&&(fulltextTag>0))
		{
		  SxmlAppendChild(metadata, IstexGetDocumentXmlByUri(uri));
		}
	      else
		{
		  SxmlAppendChild (metadata,SxmlClone(jsItem));
		}
	    }
	}
      else if  ((SxmlNodeHasName(istexElem, "fulltext")&&(fulltextTag>1)))
	{
	  SxmlNode *jsItem;
	  SxmlNode *fulltext;
	  
	  SxmlAppendChild (outputDoc, fulltext=SxmlElementCreate("fulltext"));
	  SxmlReset(istexElem);
	  while ((jsItem=SxmlNextNode(istexElem)))
	    {
	      SxmlNode *typeNode;
	      SxmlNode *uriNode;
	      char *type;
	      char *uri;
	      
	      typeNode=SxmlGetFirstChildByTagName(jsItem,"extension");
	      if(!typeNode) typeNode=SxmlGetFirstChildByTagName(jsItem,"mimetype");
	      type=SxmlLeafText(typeNode);
	      uriNode=SxmlGetFirstChildByTagName(jsItem, "uri");
	      uri=SxmlLeafText(uriNode);
	      if (strcmp(type, "tei")==0)
		{
		  SxmlAppendChild (fulltext, IstexGetDocumentTeiByUri(uri, "istex:fulltextTEI"));
		}
	      else SxmlAppendChild (fulltext, SxmlClone(jsItem));
	    }
	}
      else if  ((SxmlNodeHasName(istexElem, "enrichments")))
	{
	  SxmlNode *jsItem;
	  SxmlNode *enrichments;
	  SxmlNode *richElem;

	  SxmlAppendChild (outputDoc, enrichments=SxmlElementCreate("enrichments"));
 	  SxmlReset(istexElem);
	  while ((richElem=SxmlNextNode(istexElem)))
	    {
	      SxmlNode *richItem;
	      SxmlNode *targetRichElem;
	      SxmlAppendChild(enrichments, targetRichElem=SxmlElementCreate(SxmlNodeName(richElem)));
	      while ((richItem=SxmlNextNode(richElem)))
		{
		  SxmlNode *typeNode;
		  typeNode=SxmlGetFirstChildByTagName(richItem,"mimetype");
		  if (typeNode)
		    {
		      if (strcmp(SxmlLeafText(typeNode),"application/tei+xml")==0)
			{
			  char *uri;
			  SxmlNode *uriNode;
			  char *istexResult;
			  uriNode=SxmlGetFirstChildByTagName(richItem, "uri");
			  uri=SxmlLeafText(uriNode);
			  SxmlSetAttribute(targetRichElem, "xmlns:tei", "http://www.tei-c.org/ns/1.0");
			  istexResult=IstexGetDocumentByUri(uri);
			  if (istexResult)
			    {
			      char *beginTEI;
			      char *endTeiContent;
			      beginTEI=strstr(istexResult, "<TEI");
			      if (beginTEI)
				{
				  endTeiContent=strrchr(istexResult, '<');
				  if (strncmp(endTeiContent, "</TEI",5)!=0)
				    {
				      fprintf(stderr, "**** IstexGetCorpus  : TEI markup problem in %s\n", uri);
				      exit (EXIT_FAILURE);
				    }
				  SxmlAppendChild(targetRichElem, SxmlTextCreate(beginTEI));
				}
			      else SxmlAppendChild(targetRichElem,SxmlClone(richItem));
			    }
			  else SxmlAppendChild(targetRichElem,SxmlClone(richItem));
			}
		      else SxmlAppendChild(targetRichElem,SxmlClone(richItem));
		    }
		  else SxmlAppendChild(targetRichElem,SxmlClone(richItem));
		}
	    }
	}
      else 
	{
	  SxmlAppendChild (outputDoc, SxmlClone(istexElem));
	}	  
    }
  return outputDoc;
}
  
