/***********************************************************************
*
* Projet   : DilibPro
* Module   : SgmlPath
* Fichier  : SgmlPath.h
* Auteur   : Ducloy
* Date     : 12/93
*
************************************************************************
* 
*     Copyrigth (C) 1995 CRIN - CNRS & INRIA Lorraine
*
*************************************************************************/

#ifndef _SGML_PATH_H
#define _SGML_PATH_H

#include "SgmlInput.h"
#include "RegExp.h"

enum SgmlPath_StepCode_Enum     /* code instruction principal */
{
  SgmlPath_StepCode_Exec,

  SgmlPath_StepCode_First,
  SgmlPath_StepCode_Last,
  SgmlPath_StepCode_SonNumber,

  SgmlPath_StepCode_IterSon,
  SgmlPath_StepCode_IterData,

  SgmlPath_StepCode_IfTag,
  SgmlPath_StepCode_IfData,
  SgmlPath_StepCode_IfAttId,
  SgmlPath_StepCode_IfAttVal,

  SgmlPath_StepCode_Data_Test,
  SgmlPath_StepCode_Parent,

  SgmlPath_StepCode_GetAttVal,
};


enum SgmlPath_StepMode_Enum     /* mode d'execution de l'instruction */
{
  SgmlPath_StepMode_String,
  SgmlPath_StepMode_RegExp,
  SgmlPath_StepMode_Num
};

enum SgmlPath_StepOperator_Enum     /* mode d'execution de l'instruction */
{
  SgmlPath_StepOperator_equal,
  SgmlPath_StepOperator_GT,
  SgmlPath_StepOperator_GE,
  SgmlPath_StepOperator_LT,
  SgmlPath_StepOperator_LE
};


struct SgmlPathStepStruct
{
  enum SgmlPath_StepCode_Enum       statCode;
  enum SgmlPath_StepMode_Enum       statMode;
  enum SgmlPath_StepOperator_Enum   operator;
  struct SgmlPathStepStruct        *nextTrue;
  struct SgmlPathStepStruct        *ret;
  int                               curContext;
  int                               nContext;
  union sre 
    {
      char *s; 
      RegExp *re;
      int i;
    }                            data;
#ifdef DILIBDEBUG
  int                            numStep;
  struct SgmlPathStepStruct     *previous;
  char                          *commentStr;
#endif
};

typedef struct SgmlPathStepStruct SgmlPathStep;

#define SgmlPathStep_Code(x)            x->statCode
#define SgmlPathStep_Mode(x)            x->statMode
#define SgmlPathStep_Operator(x)        x->operator
#define SgmlPathStep_NextTrue(x)        x->nextTrue
#define SgmlPathStep_CurrentContext(x)  x->curContext
#define SgmlPathStep_NewContext(x)      x->nContext
#define SgmlPathStep_Return(x)          x->ret
#define SgmlPathStep_String(x)          x->data.s
#define SgmlPathStep_RegExp(x)          x->data.re
#define SgmlPathStep_Number(x)          x->data.i

struct SgmlPath_StepContext_Struct
{
  SgmlNode *currentNode;
  SgmlNode *currentAtt;
  char     *currentString;
};

typedef struct SgmlPath_StepContext_Struct SgmlPath_StepContext;

struct SgmlPathProg_Struct
{
  SgmlPathStep     *firstStep;
  SgmlPathStep     *lastStep;
  int               numberOfContext;
  char             *source;
#ifdef DILIBDEBUG
  int               SgmlPathProg_CurrentStep;
#endif
};

typedef struct SgmlPathProg_Struct SgmlPathProg;

/******************** le controleur d'iteration **********************/

struct SgmlPathIteratorStruct
{
  SgmlPathProg           *prog;
  SgmlNode               *tree;
  SgmlPathStep           *currentStep;
  int                     currentContext;
  SgmlPath_StepContext   *context;
};

typedef struct SgmlPathIteratorStruct SgmlPathIterator;


SgmlPathIterator *SgmlPathIteratorCreate();
SgmlPathIterator *SgmlPathIteratorInit();
SgmlNode         *SgmlPathNext();
SgmlPathProg     *SgmlPathCompile();

void SgmlPathIteratorCopyContext();
void SgmlPathIteratorFree();
void SgmlPathProgFree();

#define SgmlPathStrSearchFirst(n,str,n1)       {     \
      static   SgmlPathProg *p;                \
      static   int firstCall=0;                \
      static   SgmlPathIterator *i;            \
      if(firstCall==0)                         \
        {firstCall=1;                          \
         p=SgmlPathCompile(str);               \
         i=SgmlPathIteratorCreate(p);          \
         }                                     \
     SgmlPathIteratorInit(i,n);                \
     n1=SgmlPathNext(i) ;                      \
    }


#define SgmlPathStrSearchFirstData(n,str,str1)   {   \
      static   SgmlPathProg *p;                \
      static   int firstCall=0;                \
      static   SgmlPathIterator *i;            \
      SgmlNode *n1;                            \
      if(firstCall==0)                         \
        {firstCall=1;                          \
         p=SgmlPathCompile(str);               \
         i=SgmlPathIteratorCreate(p);          \
         }                                     \
     SgmlPathIteratorInit(i,n);               \
     n1=SgmlPathNext(i) ;                     \
     str1=SgmlLeafGetData(n1);                \
    }



#define SgmlPathStrDo(n,str,n1,block)       {                \
      static   SgmlPathProg *_DILIB_p;                        \
      static   int _DILIB_firstCall=0;                        \
      static   SgmlPathIterator *_DILIB_i;                    \
      if((_DILIB_firstCall==0))                               \
        {_DILIB_firstCall=1;                                  \
         _DILIB_p=SgmlPathCompile(str);                       \
         _DILIB_i=SgmlPathIteratorCreate(_DILIB_p);           \
         }                                                    \
     SgmlPathIteratorInit(_DILIB_i,n);                        \
      while ((n1=SgmlPathNext(_DILIB_i)))block;               \
    }

#define SgmlPathStrDoData(n,str,str1,block)       { \
      static   SgmlPathProg *p;                \
      static   int firstCall=0;                \
      static   SgmlPathIterator *i;            \
      SgmlNode *n1;                            \
      if(firstCall==0)                         \
        {firstCall=1;                          \
         p=SgmlPathCompile(str);               \
         i=SgmlPathIteratorCreate(p);           \
         }                                     \
     SgmlPathIteratorInit(i,n);              \
      while (n1=SgmlPathNext(i))               \
         { str1=SgmlLeafGetData(n1); block};   \
    }


/*                listes de path         */

struct SgmlPathElemList_struct
{
  SgmlPathProg                   *prog;
  SgmlPathIterator               *iter;
  struct SgmlPathElemList_struct *prev;
  struct SgmlPathElemList_struct *next;
};

struct SgmlPathList_struct
{
  struct SgmlPathElemList_struct *first;
  struct SgmlPathElemList_struct *last;
  struct SgmlPathElemList_struct *current;
  SgmlNode *root;
};
typedef struct SgmlPathList_struct SgmlPathList;

SgmlPathList *SgmlPathList_Create();
SgmlPathList *SgmlPathList_Insert();
SgmlPathList *SgmlPathList_Init();
void          SgmlPathList_Free();
SgmlPathList *SgmlPathList_FromString();
SgmlNode *SgmlPathList_Next();
SgmlPathList * SgmlPathList_GetOpt();

#endif /*   _SGML_PATH_H  */



