/***********************************************************************
*
*      Projet  : DilibSxml
*      Module  : SxmlProc
*      Fichier : SxmlC.c
*      Auteur  : J. Ducloy
*
*      Derniere mise a jour Decembre 2000
*      $Id: SxmlC.c,v 1.2 2003/06/13 16:32:11 parmentf Exp $
*
************************************************************************
* 
*     Copyright (C) 2001 CNRS INIST
*
************************************************************************/
/**
   @file

   @brief All functions included in the c: namespace

   This file implements C functions included in the c: namespace:
   - c:printf
   
   and the tools around: a function to create parameters to c:printf,
   and the one that adds all the functions of the c: namespace in a
   SxmlMemory.

   @since Dec 2000
   @author &copy; 2001 INIST-CNRS
   @author Jacques DUCLOY
 */
/*
   @addtogroup SxmlProc
   @{
 */

#include "SxmlC.h"
#include "Buffer.h"
#include <ctype.h>


/**
   Creates a "c:printf" step from @a str1 with "@" specifications.

   The specifications contain a string with any character, except that
   "@" have a specific meaning: the two characters following "@"
   constitute a variable name. If you use variable names longer than
   2, only the two first characters will be taken into account, the
   others will be in the format string.

   Example of @a str1:
   @code
   "To display the variable va: @va\n"
   @endcode

   @param str1	specifications of what is to be printed
   @return an instruction "c:printf"
   @todo Check the example
   @see SxmlCProcPrintf
 */
SxmlNode *SxmlCPrintfFromStringVar(char *str1)
{
  SxmlNode       *stepNode;
  static Buffer *bufTrad = NULL;
  static Buffer *bufName = NULL;
  char          *str2;
  char           c1;

  stepNode=SxmlElementCreate("c:printf");
  str2=str1;
  if (!(bufTrad))bufTrad=BufferCreate(8,8);
  if (!(bufName))bufName=BufferCreate(8,8);
  BufferReset(bufTrad);
  while((c1=*str1++))
    {
      if(c1=='@')
	{
	  SxmlNode *var1;
	  BufferStrcat(bufTrad,"%s");
	  BufferStrcpy(bufName,"#");
	  if (isdigit(*str1))
	    {
	      BufferCharCat(bufName,*str1++);
	    }
	  else
	    {
	      BufferCharCat(bufName,*str1++);
	      BufferCharCat(bufName,*str1++);
	    }
	  SxmlAppendChild(stepNode,var1=SxmlElementCreate("dom:getVar"));
	  SxmlSetAttribute(var1,"name",BufferString(bufName));
	}
      else BufferCharCat(bufTrad,c1);
    }
  SxmlAddFirstChild(stepNode, SxmlTextCreate(BufferString(bufTrad)));
  return stepNode;
}


/**
   @brief Prints @a x1 on standard output.

   Evaluates @a x1 in @a p1, and prints all text to the standard
   output, except variables that can be specified by putting a "@" in
   front of their names (but they can't be longer than 2 characters).

   @param p1
   @param x1 a paramater to @c c:printf that can be done by @ref
   SxmlCPrintfFromStringVar
   @return @a x1
   @see SxmlCPrintfFromStringVar
 */
SxmlNode *SxmlCProcPrintf(SxmlProcessor *p1,
			SxmlNode *x1)
{
  char    *format1;
  SxmlNode *x2;
  char    *posPerCent;
  static Buffer *bufZone=NULL;

  if(!bufZone)bufZone=BufferCreate(10,10);
  else BufferReset(bufZone);
  format1=SxmlProcessorEvalToString(p1,SxmlFirstChild(x1));
  x2=SxmlFirstChild(x1);
  if ((posPerCent=strchr(format1,'%')))
    {
      char *pc2;
      char *toPrint;
      BufferStrncpy(bufZone,format1,posPerCent-format1);
      printf("%s",BufferString(bufZone));
      pc2=posPerCent+1;
      while((pc2=strchr(pc2,'%')))
	{
	  BufferStrncpy(bufZone, posPerCent, pc2-posPerCent);
	  if((x2=SxmlNextSibling(x2))
	     &&(toPrint=SxmlProcessorEvalToString(p1,x2)))
	    {
	      printf(BufferString(bufZone),toPrint);
	      posPerCent=pc2;
	      pc2=posPerCent+1;
	    }
	  else printf(BufferString(bufZone),"(null)");
	}
      if((x2=SxmlNextSibling(x2))
	 &&(toPrint=SxmlProcessorEvalToString(p1,x2)))
	    {
	      printf(posPerCent,toPrint);
	    }
      else printf(posPerCent,"(null)");
    }
  else
    {
      printf("%s",format1);
    }
  return x1;
}

/**
   Adds a function to @a p1: "c:printf".

   @param p1
   @return the modified @a p1
   @see SxmlCProcPrintf
 */
SxmlMemory *SxmlCMemoryInit(SxmlMemory *p1)
{
  SxmlMemorySetXmlBuiltin(p1,"c:printf"         ,SxmlCProcPrintf);
  return p1;
}

/*
   @}
 */
